<?php

/*
 +-----------------------------------------------------------------------+
 | Forward Module for RoundCube                                          |
 |                                                                       |
 | Copyright (C) 2009 Boris HUISGEN <bhuisgen@hbis.fr>                   |
 | Copyright (C) 2011 Vicente MONROIG <vmonroig@digitaldisseny.com>      |
 | Licensed under the GNU GPL                                            |
 +-----------------------------------------------------------------------+
 */

define ('PLUGIN_SUCCESS', 0);
define ('PLUGIN_ERROR_DEFAULT', 1);
define ('PLUGIN_ERROR_CONNECT', 2);
define ('PLUGIN_ERROR_PROCESS', 3);

class forward extends rcube_plugin
{
	public $task = 'settings';
	private $rc;
	private $obj;

	/*
	 * Initializes the plugin.
	 */
	public function init()
	{
		$rcmail = rcmail::get_instance();
		$this->rc = &$rcmail;
		$this->add_texts('localization/', true);

		$this->rc->output->add_label('forward');
		$this->register_action('plugin.forward', array($this, 'forward_init'));
		$this->register_action('plugin.forward-save', array($this, 'forward_save'));
		$this->include_script('forward.js');

		$this->load_config();

		require_once ($this->home . '/lib/rcube_forward.php');
		$this->obj = new rcube_forward();
	}

	/*
	 * Plugin initialization function.
	 */
	public function forward_init()
	{
		$this->read_data();

		$this->register_handler('plugin.body', array($this, 'forward_form'));
		$this->rc->output->set_pagetitle($this->gettext('forward'));
		$this->rc->output->send('plugin');
	}

	/*
	 * Plugin save function.
	 */
	public function forward_save()
	{
		$this->write_data();

		$this->register_handler('plugin.body', array($this, 'forward_form'));
		$this->rc->output->set_pagetitle($this->gettext('forward'));
		rcmail_overwrite_action('plugin.forward');
		$this->rc->output->send('plugin');
	}

	/*
	 * Plugin UI form function.
	 */
	public function forward_form()
	{
		$table = new html_table(array('cols' => 2));

		$field_id = 'forwardenable';
		$input_forwardenable = new html_checkbox(array('name' => '_forwardenable', 'id' => $field_id, 'value' => 1));
		$table->add('title', html::label($field_id, Q($this->gettext('forwardenable'))));
		$table->add(null, $input_forwardenable->show($this->obj->is_forward_enable() ? 1 : 0));

		$field_id = 'forwardaddresses';
    	$text_forwardaddresses = new html_textarea(array('name' => '_forwardaddresses', 'id' => $field_id, 'spellcheck' => 1, 'rows' => 6, 'cols' => 40));
		$table->add('title', html::label($field_id, Q($this->gettext('forwardaddresses'))));
		$table->add(null, $text_forwardaddresses->show($this->obj->get_forward_addresses()));

		$field_id = 'origforwardaddresses';
    	$hidden_origforwardaddresses = new html_hiddenfield(array('name' => '_origforwardaddresses', 'id' => $field_id, 'value' => $this->obj->get_orig_forward_addresses()));
		$table->add(null, $hidden_origforwardaddresses->show($this->obj->get_orig_forward_addresses()));

		$out = html::div(array('class' => "box"), html::div(array('id' => "prefs-title", 'class' => 'boxtitle'), $this->gettext('forward')) . html::div(array('class' => "boxcontent"), $table->show() . html::p(null, $this->rc->output->button(array('command' => 'plugin.forward-save', 'type' => 'input', 'class' => 'button mainaction', 'label' => 'save')))));

		$this->rc->output->add_gui_object('forwardform', 'forward-form');

		return $this->rc->output->form_tag(array('id' => 'forward-form', 'name' => 'forward-form', 'method' => 'post', 'action' => './?_task=settings&_action=plugin.forward-save'), $out);
	}

	/*
	 * Reads plugin data.
	 */
	public function read_data()
	{
		$driver = $this->home . '/lib/drivers/' . $this->rc->config->get('forward_driver', 'plesk').'.php';

		if (!is_readable($driver))
		{
			raise_error(array('code' => 600, 'type' => 'php', 'file' => __FILE__, 'message' => "forward plugin: Unable to open driver file $driver"), true, false);

			return $this->gettext('internalerror');
		}

		require_once($driver);

		if (!function_exists('forward_read'))
		{
			raise_error(array('code' => 600, 'type' => 'php', 'file' => __FILE__, 'message' => "forward plugin: Broken driver: $driver"), true, false);

			return $this->gettext('internalerror');
		}

		$data['username'] = $this->obj->get_username();
		$data['email'] = $this->obj->get_email();
		$data['email_local'] = $this->obj->get_email_local();
		$data['email_domain'] = $this->obj->get_email_domain();
		$data['forward_enable'] = $this->obj->is_forward_enable();
		$data['forward_addresses'] = $this->obj->get_forward_addresses();
		$data['orig_forward_addresses'] = $this->obj->get_orig_forward_addresses();

		$ret = forward_read ($data);

		switch ($ret)
		{
			case PLUGIN_ERROR_DEFAULT:
				{
					$this->rc->output->command('display_message', $this->gettext('forwarddriverdefaulterror'), 'error');

					return FALSE;
				}

			case PLUGIN_ERROR_CONNECT:
				{
					$this->rc->output->command('display_message', $this->gettext('forwarddriverconnecterror'), 'error');

					return FALSE;
				}

			case PLUGIN_ERROR_PROCESS:
				{
					$this->rc->output->command('display_message', $this->gettext('forwarddriverprocesserror'), 'error');

					return FALSE;
				}

			case PLUGIN_SUCCESS:
			default:
				{
					break;
				}
		}

		if (isset($data['email']))
		{
			$this->obj->set_email($data['email']);
		}

		if (isset($data['email_local']))
		{
			$this->obj->set_email_local($data['email_local']);
		}

		if (isset($data['email_domain']))
		{
			$this->obj->set_email_domain($data['email_domain']);
		}

		if (isset($data['forward_enable']))
		{
			$this->obj->set_forward_enable($data['forward_enable']);
		}

		if (isset($data['forward_addresses']) && (strlen ($data['forward_addresses']) > 0))
		{
			$this->obj->set_forward_addresses($data['forward_addresses']);
		}

		if (isset($data['orig_forward_addresses']))
		{
			$this->obj->set_orig_forward_addresses($data['orig_forward_addresses']);
		}

		return TRUE;
	}

	/*
	 * Writes plugin data.
	 */
	public function write_data()
	{
		if (get_input_value('_forwardenable', RCUBE_INPUT_POST))
			$this->obj->set_forward_enable(TRUE);
		else
			$this->obj->set_forward_enable(FALSE);

		$addresses = get_input_value('_forwardaddresses', RCUBE_INPUT_POST);
		if ($this->obj->is_forward_enable() && (!is_string($addresses) || (strlen($addresses) == 0)))
		{
			$this->rc->output->command('display_message', $this->gettext('forwardnoaddresses'), 'error');	
			
			return FALSE;
		}
		$this->obj->set_forward_addresses($addresses);

		$origaddresses = get_input_value('_origforwardaddresses', RCUBE_INPUT_POST);
   		$this->obj->set_orig_forward_addresses($origaddresses);

		$driver = $this->home . '/lib/drivers/' . $this->rc->config->get('forward_driver', 'plesk').'.php';

		if (!is_readable($driver))
		{
			raise_error(array('code' => 600, 'type' => 'php', 'file' => __FILE__, 'message' => "forward plugin: Unable to open driver file $driver"), true, false);

			return $this->gettext('internalerror');
		}

		require_once($driver);

		if (!function_exists('forward_write'))
		{
			raise_error(array('code' => 600, 'type' => 'php', 'file' => __FILE__, 'message' => "forward plugin: Broken driver: $driver"), true, false);

			return $this->gettext('internalerror');
		}

		$data['username'] = $this->obj->get_username();
		$data['email'] = $this->obj->get_email();
		$data['email_local'] = $this->obj->get_email_local();
		$data['email_domain'] = $this->obj->get_email_domain();
		$data['forward_enable'] = $this->obj->is_forward_enable();
		$data['forward_addresses'] = $this->obj->get_forward_addresses();
		$data['orig_forward_addresses'] = $this->obj->get_orig_forward_addresses();

		$ret = forward_write ($data);

		switch ($ret)
		{
			case PLUGIN_ERROR_DEFAULT:
				{
					$this->rc->output->command('display_message', $this->gettext('forwarddriverdefaulterror'), 'error');

					return FALSE;
				}

			case PLUGIN_ERROR_CONNECT:
				{
					$this->rc->output->command('display_message', $this->gettext('forwarddriverconnecterror'), 'error');

					return FALSE;
				}

			case PLUGIN_ERROR_PROCESS:
				{
					$this->rc->output->command('display_message', $this->gettext('forwarddriverprocesserror'), 'error');

					return FALSE;
				}

			case PLUGIN_SUCCESS:
			default:
				{
					$this->rc->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');

					break;
				}
		}

		if (isset($data['email']))
		{
			$this->obj->set_email($data['email']);
		}

		if (isset($data['email_local']))
		{
			$this->obj->set_email_local($data['email_local']);
		}

		if (isset($data['email_domain']))
		{
			$this->obj->set_email_domain($data['email_domain']);
		}

		if (isset($data['forward_enable']))
		{
			$this->obj->set_forward_enable($data['forward_enable']);
		}

		if (isset($data['forward_addresses']))
		{
			$this->obj->set_forward_addresses($data['forward_addresses']);
		}

		if (isset($data['orig_forward_addresses']))
		{
			$this->obj->set_orig_forward_addresses($data['orig_forward_addresses']);
		}

		return TRUE;
	}
	
}
