<?php

/*
 +-----------------------------------------------------------------------+
 | lib/drivers/plesk.php                                                 |
 |                                                                       |
 | Copyright (C) 2009 Boris HUISGEN <bhuisgen@hbis.fr>                   |
 | Copyright (C) 2011 Vicente MONROIG <vmonroig@digitaldisseny.com>      |
 | Licensed under the GNU GPL                                            |
 +-----------------------------------------------------------------------+

 +-----------------------------------------------------------------------+
   Please, note:
   Needs access for apache user in /etc/sudoers. For example:
   www-data ALL=NOPASSWD: /opt/psa/bin/mail
 +-----------------------------------------------------------------------+
 */

// Array_search with partial matches and optional search by key
// http://www.php.net/manual/en/function.array-search.php#95926
function array_find($needle, $haystack, $search_keys = false)
{
    if (!is_array($haystack))
        return false;
    foreach ($haystack as $key=>$value)
    {
        $what = ($search_keys) ? $key : $value;
        if (strpos($what, $needle) !== false)
            return $key;
    }
    return false;
}

// Parse output from Plesk CLI autoresponder
function parse_output($output, $strkey)
{
    $key = array_find($strkey, $output);
    if ($key !== false)
        return substr($output[$key], 20);
    else
        return false;
}


/*
 * Read driver function.
 *
 * @param array $data the array of data to get and set.
 *
 * @return integer the status code.
 */

function forward_read(array &$data)
{
	$rcmail = rcmail::get_instance();

    $email = escapeshellcmd($data['email']);

    $cmd = "sudo /opt/psa/bin/mail -i $email";
    exec($cmd . ' 2>&1', $output, $returnvalue);

    if ($returnvalue !== 0)
    {
        $stroutput = implode(' ', $output);
        raise_error(array(
            'code' => 600,
            'type' => 'php',
            'file' => __FILE__, 'line' => __LINE__,
            'message' => "forward plugin: Unable to execute $cmd ($stroutput, $returnvalue)"
            ), true, false);
        return PLUGIN_ERROR_CONNECT;
    }

    $status = parse_output($output, "Mailgroup:");
    if ($status == "true")
        $data['forward_enable'] = true;
    $data['orig_forward_addresses'] = str_replace(" ", ",", parse_output($output, "Group member(s):"));
    $data['forward_addresses'] = str_replace(",", "\r\n", $data['orig_forward_addresses']);

	return PLUGIN_SUCCESS;
}


// Call exec and raise exception if problems detected
function tryexec($cmd)
{
    exec($cmd . ' 2>&1', $output, $returnvalue);
    if ($returnvalue == 0) 
        return TRUE;
    else
    {
        $stroutput = implode(' ', $output);
        raise_error(array(
            'code' => 600,
            'type' => 'php',
            'file' => __FILE__, 'line' => __LINE__,
            'message' => "forward plugin: Unable to execute $cmd ($stroutput, $returnvalue)"
            ), true, false);
        return FALSE;
    }
}

/*
 * Write driver function.
 *
 * @param array $data the array of data to get and set.
 *
 * @return integer the status code.
 */
function forward_write(array &$data)
{
	$rcmail = rcmail::get_instance();

    $email = escapeshellcmd($data['email']);
    if ($data['forward_enable'])
        $status = "true";
    else
        $status = "false";
    $new_addresses = explode("\r\n", $data['forward_addresses']);
    $orig_addresses = explode(",", $data['orig_forward_addresses']);
    $del_addresses = array_diff($orig_addresses, $new_addresses);

    if ($data['forward_enable'])
    {
        $stradd_addresses = implode(",", $new_addresses);
        $strdel_addresses = implode(",", $del_addresses);
    }
    else
    {
        // Delete all, as Plesk does not delete, but does not inform later if disabled, neither
        $stradd_addresses = "";
        $strdel_addresses = implode(",", $orig_addresses);
    }

    // Call CLI mail in two stages, beacuse can't nest add and del forwarding addresses in same call
    if (is_string($strdel_addresses) && (strlen($strdel_addresses) > 0))
    {
        $cmd = sprintf("sudo /opt/psa/bin/mail -u %s -forwarding %s", $email, $status);
        $cmd = $cmd . " -forwarding-addresses del:" . $strdel_addresses;
        if (tryexec($cmd))
        {
            if (is_string($stradd_addresses) && (strlen($stradd_addresses) > 0))
            {
                $cmd = sprintf("sudo /opt/psa/bin/mail -u %s -forwarding %s", $email, $status);
                $cmd = $cmd . " -forwarding-addresses add:" . $stradd_addresses;
                if (tryexec($cmd))
                    return PLUGIN_SUCCESS;
            }    
            else
                return PLUGIN_SUCCESS;                        
        }
    }
    else
    {
        if (is_string($stradd_addresses) && (strlen($stradd_addresses) > 0))
        {
            $cmd = sprintf("sudo /opt/psa/bin/mail -u %s -forwarding %s", $email, $status);
            $cmd = $cmd . " -forwarding-addresses add:" . $stradd_addresses;
            if (tryexec($cmd))
                return PLUGIN_SUCCESS;
        }        
    }

    return PLUGIN_ERROR_PROCESS;
}
