<?php

/*
 +-----------------------------------------------------------------------+
 | lib/rcube_forward.php                                                 |
 |                                                                       |
 | Copyright (C) 2009 Boris HUISGEN <bhuisgen@hbis.fr>                   |
 | Copyright (C) 2011 Vicente MONROIG <vmonroig@digitaldisseny.com>      |
 | Licensed under the GNU GPL                                            |
 +-----------------------------------------------------------------------+
 */

class rcube_forward
{
	public $username = '';
	public $email = '';
	public $email_local = '';
	public $email_domain =  '';
	public $forward_enable = FALSE;
	public $forward_addresses = '';
	public $orig_forward_addresses = '';

	/**
	 * Constructor of the class.
	 */
	public function __construct()
	{
		$this->init();
	}
	
	/*
	 * Initialize the object.
	 */
	private function init()
	{
		$this->username = rcmail::get_instance()->user->get_username();
		
	    $parts = explode('@', $this->username);
	    if (count($parts) >= 2)
	    {
	       $this->email = $this->username;
	       $this->email_local = $parts[0];
	       $this->email_domain = $parts[1] ;
	    }
	}
	
	/*
	 * Gets the username.
	 *
	 * @return string the username.
	 */		
	public function get_username()
	{
		return $this->username;
	}
	
	/*
	 * Gets the full email of the user.
	 *
	 * @return string the email of the user.
	 */			
	public function get_email()
	{	    
	    return $this->email;
    }
	
	/*
	 * Gets the email local part of the user.
	 *
	 * @return string the email local part.
	 */		
	public function get_email_local()
	{    
	    return $this->email_local;
    }

	/*
	 * Gets the email domain of the user.
	 *
	 * @return string the email domain.
	 */			
	public function get_email_domain()
	{	    
	    return $this->email_domain;
    }

	/*
	 * Checks if the forward is enabled.
	 *
	 * @return boolean TRUE if forward is enabled; FALSE otherwise.
	 */
	public function is_forward_enable ()
	{
		return $this->forward_enable;
	}

	/*
	 * Gets the forward addresses.
	 *
	 * @return string the forward addresses, one per line.
	 */
	public function get_forward_addresses()
	{
		return $this->forward_addresses;
	}
	
	/*
	 * Gets the original forward addresses.
	 *
	 * @return string the original forward addresses, one per line.
	 */
	public function get_orig_forward_addresses()
	{
		return $this->orig_forward_addresses;
	}
	
	/*
	 * Sets the email of the user
	 *
	 * @param string $email the email.
	 */
	public function set_email($email)
	{
		$this->email = $email;
	}
	
	/*
	 * Sets the email local part of the user
	 *
	 * @param string $local the local part of the email.
	 */
	public function set_email_local($local)
	{
		$this->email_local = $local;
	}
	
	/*
	 * Sets the email domain part of the user
	 *
	 * @param string $local the domain part of the email.
	 */
	public function set_email_domain($domain)
	{
		$this->email_domain = $domain;
	}
	
	/*
	 * Enables or disables the forward.
	 *
	 * @param boolean the flag.
	 */
	public function set_forward_enable($flag)
	{
		$this->forward_enable = $flag;
	}

	/*
	 * Sets the forward addresses.
	 *
	 * @param string $addresses the forward addresses, one per line.
	 */
	public function set_forward_addresses($addresses)
	{
		$this->forward_addresses = $addresses;
	}

	/*
	 * Sets the original forward addresses.
	 *
	 * @param string $addresses the original forward addresses, one per line.
	 */
	public function set_orig_forward_addresses($addresses)
	{
		$this->orig_forward_addresses = $addresses;
	}
	
}
